<?php // $Id: block_quickmail.php,v 1.9 2006/09/30 20:58:08 mark-nielsen Exp $
/**
 * Quickmail - Allows teachers and students to email one another
 *      at a course level.  Also supports group mode so students
 *      can only email their group members if desired.  Both group
 *      mode and student access to Quickmail are configurable by
 *      editing a Quickmail instance.
 *
 * @author Mark Nielsen
 * @version $Id: block_quickmail.php,v 1.9 2006/09/30 20:58:08 mark-nielsen Exp $
 * @package quickmail
 **/ 

/**
 * This is the Quickmail block class.  Contains the necessary
 * functions for a Moodle block.  Has some extra functions as well
 * to increase its flexibility and useability
 *
 * @package quickmail
 * @todo Make a global config so that admins can set the defaults (default for student (yes/no) default for groupmode (select a groupmode or use the courses groupmode)) NOTE: make sure email.php and emaillog.php use the global config settings
 **/
class block_quickmail extends block_list {
    
    /**
     * Sets the block name and version number
     *
     * @return void
     **/
    function init() {
        $this->title = get_string('blockname', 'block_quickmail');
        $this->version = 2006021500;  // YYYYMMDDXX
    }
    
    /**
     * Gets the contents of the block (course view)
     *
     * @return object An object with an array of items, an array of icons, and a string for the footer
     **/
    function get_content() {
        global $USER, $CFG;

        if($this->content !== NULL) {
            return $this->content;
        }
        
        $this->content = new stdClass;
        $this->content->footer = '';
        $this->content->items = array();
        $this->content->icons = array();
        
        if (empty($this->instance) or !$this->check_permission()) {
            return $this->content;
        }
                
    /// link to composing an email
        $this->content->items[] = '<a href="'.$CFG->wwwroot.'/blocks/quickmail/email.php?id='.$this->course->id.'&amp;instanceid='.$this->instance->id.'">'.
                                    get_string('compose', 'block_quickmail').'</a>';

        $this->content->icons[] = '<img src="'.$CFG->pixpath.'/i/email.gif" height="16" width="16" alt="'.get_string('email').'" />';
        
    /// link to history log
        $this->content->items[] = '<a href="'.$CFG->wwwroot.'/blocks/quickmail/emaillog.php?id='.$this->course->id.'&amp;instanceid='.$this->instance->id.'">'.
                                    get_string('history', 'block_quickmail').'</a>';

        $this->content->icons[] = '<img src="'.$CFG->pixpath.'/t/log.gif" height="14" width="14" alt="'.get_string('log').'" />';
    
    /// link to config for teachers
        if (isteacher($this->instance->pageid)) {
            $this->content->footer = "<a href=\"$CFG->wwwroot/course/view.php?id={$this->instance->pageid}&amp;instanceid={$this->instance->id}&amp;sesskey=$USER->sesskey&amp;blockaction=config\">".
                            get_string('settings').'...</a>';
        }
        
        return $this->content;
    }
    
    /**
     * Loads the course and defaults.
     *
     * @return void
     **/
    function specialization() {
        $this->course = get_record('course', 'id', $this->instance->pageid);
        $this->load_defaults();
    }
    
    /**
     * Allows the block to be configurable at an instance level.
     *
     * @return boolean
     **/
    function instance_allow_config() {
        return true;
    }
    
    /**
     * Check to make sure that the current user is allowed to use Quickmail.
     *
     * Permissions:
     *          Teacher/Admin:  Always allow
     *          Student:        If allowed by instance
     *          Guest/Other:    Never allow
     * @return boolean True for access / False for denied
     **/
    function check_permission() {
        if (isteacher($this->instance->pageid)) {
            return true;
        } else if ($this->config->allowstudents) {
            return isstudent($this->instance->pageid);
        } else {
            return false;
        }
    }

    /**
     * Get the groupmode of Quickmail.  This function pays
     * attention to the course group mode force.
     *
     * @return int The group mode of the block
     **/
    function groupmode() {
        if ($this->course->groupmodeforce) {
            return $this->course->groupmode;
        } else {
            return $this->config->groupmode;
        }
    }

    /**
     * Loads default config data when config is empty (that way we know it exists).
     *
     * Defaults:
     *      group mode           = course group mode
     *      allow student access = yes
     * @return boolean
     * @todo Make a global config so that admins can set the defaults (default for student (yes/no) default for groupmode (select a groupmode or use the courses groupmode))  NOTE: make sure email.php and emaillog.php use the global config settings
     **/
    function load_defaults() {
        if (empty($this->config)) {
        /// blank config
            $defaults = new stdClass;
            $defaults->groupmode = $this->course->groupmode;
            $defaults->allowstudents = 1;

            return $this->instance_config_save($defaults);
        }
        
        return true;
    }
}
?>
